@@ -459,7 +459,7 @@ int mainGtU(EState* state,
  * usually small, typically <= 20.
  */
 static
-const uint32_t incs[14] = {
+const uint32_t incs[14] ALIGN4 = {
 	1, 4, 13, 40, 121, 364, 1093, 3280,
 	9841, 29524, 88573, 265720,
 	797161, 2391484
