@@ -19,16 +19,8 @@
 //config:	touch is used to create or change the access and/or
 //config:	modification timestamp of specified files.
 //config:
-//config:config FEATURE_TOUCH_NODEREF
-//config:	bool "Add support for -h"
-//config:	default y
-//config:	depends on TOUCH
-//config:	help
-//config:	Enable touch to have the -h option.
-//config:	This requires libc support for lutimes() function.
-//config:
 //config:config FEATURE_TOUCH_SUSV3
-//config:	bool "Add support for SUSV3 features (-d -t -r)"
+//config:	bool "Add support for SUSV3 features (-a -d -m -t -r)"
 //config:	default y
 //config:	depends on TOUCH
 //config:	help
@@ -38,18 +30,17 @@
 
 //kbuild:lib-$(CONFIG_TOUCH) += touch.o
 
-/* BB_AUDIT SUSv3 _NOT_ compliant -- options -a, -m not supported. */
-/* http://www.opengroup.org/onlinepubs/007904975/utilities/touch.html */
-
 //usage:#define touch_trivial_usage
-//usage:       "[-c]" IF_FEATURE_TOUCH_SUSV3(" [-d DATE] [-t DATE] [-r FILE]") " FILE..."
+//usage:       "[-ch" IF_FEATURE_TOUCH_SUSV3("am") "]"
+//usage:       IF_FEATURE_TOUCH_SUSV3(" [-d DATE] [-t DATE] [-r FILE]")
+//usage:       " FILE..."
 //usage:#define touch_full_usage "\n\n"
-//usage:       "Update the last-modified date on the given FILE[s]\n"
+//usage:       "Update mtime of FILEs\n"
 //usage:     "\n	-c	Don't create files"
-//usage:	IF_FEATURE_TOUCH_NODEREF(
 //usage:     "\n	-h	Don't follow links"
-//usage:	)
 //usage:	IF_FEATURE_TOUCH_SUSV3(
+//usage:     "\n	-a	Change only atime"
+//usage:     "\n	-m	Change only mtime"
 //usage:     "\n	-d DT	Date/time to use"
 //usage:     "\n	-t DT	Date/time to use"
 //usage:     "\n	-r FILE	Use FILE's date/time"
@@ -85,70 +76,68 @@ int touch_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int touch_main(int argc UNUSED_PARAM, char **argv)
 {
 	int fd;
-	int status = EXIT_SUCCESS;
 	int opts;
-	enum {
-		OPT_c = (1 << 0),
-		OPT_r = (1 << 1) * ENABLE_FEATURE_TOUCH_SUSV3,
-		OPT_d = (1 << 2) * ENABLE_FEATURE_TOUCH_SUSV3,
-		OPT_t = (1 << 3) * ENABLE_FEATURE_TOUCH_SUSV3,
-		OPT_h = (1 << 4) * ENABLE_FEATURE_TOUCH_NODEREF,
-	};
+	smalluint status = EXIT_SUCCESS;
 #if ENABLE_FEATURE_TOUCH_SUSV3
-# if ENABLE_LONG_OPTS
-	static const char touch_longopts[] ALIGN1 =
-		/* name, has_arg, val */
-		"no-create\0"         No_argument       "c"
-		"reference\0"         Required_argument "r"
-		"date\0"              Required_argument "d"
-		IF_FEATURE_TOUCH_NODEREF("no-dereference\0" No_argument "h")
-	;
-#  define GETOPT32 getopt32long
-#  define LONGOPTS ,touch_longopts
-# else
-#  define GETOPT32 getopt32
-#  define LONGOPTS
-# endif
-	char *reference_file = NULL;
-	char *date_str = NULL;
-	struct timeval timebuf[2];
-	timebuf[1].tv_usec = timebuf[0].tv_usec = 0;
+	char *reference_file;
+	char *date_str;
+	/* timebuf[0] is atime, timebuf[1] is mtime */
+	struct timespec timebuf[2];
 #else
 # define reference_file NULL
 # define date_str       NULL
-# define timebuf        ((struct timeval*)NULL)
-# define GETOPT32 getopt32
-# define LONGOPTS
+# define timebuf        ((struct timespec*)NULL)
 #endif
 
+	enum {
+		OPT_c = (1 << 0),
+		OPT_h = (1 << 1),
+		OPT_r = (1 << 2) * ENABLE_FEATURE_TOUCH_SUSV3,
+		OPT_d = (1 << 3) * ENABLE_FEATURE_TOUCH_SUSV3,
+		OPT_t = (1 << 4) * ENABLE_FEATURE_TOUCH_SUSV3,
+		OPT_a = (1 << 5) * ENABLE_FEATURE_TOUCH_SUSV3,
+		OPT_m = (1 << 6) * ENABLE_FEATURE_TOUCH_SUSV3,
+	};
+#if ENABLE_LONG_OPTS
+	static const char touch_longopts[] ALIGN1 =
+		/* name, has_arg, val */
+		"no-create\0"        No_argument       "c"
+		"no-dereference\0"   No_argument       "h"
+		IF_FEATURE_TOUCH_SUSV3("reference\0"        Required_argument "r")
+		IF_FEATURE_TOUCH_SUSV3("date\0"             Required_argument "d")
+	;
+#endif
 	/* -d and -t both set time. In coreutils,
 	 * accepted data format differs a bit between -d and -t.
-	 * We accept the same formats for both */
-	opts = GETOPT32(argv, "c" IF_FEATURE_TOUCH_SUSV3("r:d:t:")
-				IF_FEATURE_TOUCH_NODEREF("h")
-				/*ignored:*/ "fma"
-				LONGOPTS
-				IF_FEATURE_TOUCH_SUSV3(, &reference_file)
-				IF_FEATURE_TOUCH_SUSV3(, &date_str)
-				IF_FEATURE_TOUCH_SUSV3(, &date_str)
+	 * We accept the same formats for both
+	 */
+	opts = getopt32long(argv, "^"
+		"ch"
+		IF_FEATURE_TOUCH_SUSV3("r:d:t:am")
+		/*ignored:*/ "f" IF_NOT_FEATURE_TOUCH_SUSV3("am")
+		"\0" /* opt_complementary: */
+		/* at least one arg: */ "-1"
+		/* coreutils forbids -r and -t at once: */ IF_FEATURE_TOUCH_SUSV3(":r--t:t--r")
+		/* but allows these combinations: "r--d:d--r:t--d:d--t" */
+		, touch_longopts
+#if ENABLE_FEATURE_TOUCH_SUSV3
+		, &reference_file
+		, &date_str
+		, &date_str
+#endif
 	);
 
-	argv += optind;
-	if (!*argv) {
-		bb_show_usage();
-	}
-
-	if (reference_file) {
+#if ENABLE_FEATURE_TOUCH_SUSV3
+	timebuf[0].tv_nsec = timebuf[1].tv_nsec = UTIME_NOW;
+	if (opts & OPT_r) {
 		struct stat stbuf;
 		xstat(reference_file, &stbuf);
-		timebuf[1].tv_sec = timebuf[0].tv_sec = stbuf.st_mtime;
-		/* Can use .st_mtim.tv_nsec
-		 * (or is it .st_mtimensec?? see date.c)
-		 * to set microseconds too.
-		 */
+		timebuf[0].tv_sec = stbuf.st_atime;
+		timebuf[1].tv_sec = stbuf.st_mtime;
+		timebuf[0].tv_nsec = stbuf.st_atim.tv_nsec;
+		timebuf[1].tv_nsec = stbuf.st_mtim.tv_nsec;
 	}
-
-	if (date_str) {
+	if (opts & (OPT_d|OPT_t)) {
 		struct tm tm_time;
 		time_t t;
 
@@ -163,15 +152,21 @@ int touch_main(int argc UNUSED_PARAM, char **argv)
 		t = validate_tm_time(date_str, &tm_time);
 
 		timebuf[1].tv_sec = timebuf[0].tv_sec = t;
+		timebuf[1].tv_nsec = timebuf[0].tv_nsec = 0;
 	}
-
-	do {
-		int result;
-		result = (
-#if ENABLE_FEATURE_TOUCH_NODEREF
-			(opts & OPT_h) ? lutimes :
+	/* If both -a and -m specified, both times should be set.
+	 * IOW: set OMIT only if one, not both, of them is given!
+	 */
+	if ((opts & (OPT_a|OPT_m)) == OPT_a)
+		timebuf[1].tv_nsec = UTIME_OMIT;
+	if ((opts & (OPT_a|OPT_m)) == OPT_m)
+		timebuf[0].tv_nsec = UTIME_OMIT;
 #endif
-			utimes)(*argv, (reference_file || date_str) ? timebuf : NULL);
+
+	argv += optind;
+	do {
+		int result = utimensat(AT_FDCWD, *argv, timebuf,
+				(opts & OPT_h) ? AT_SYMLINK_NOFOLLOW : 0);
 		if (result != 0) {
 			if (errno == ENOENT) { /* no such file? */
 				if (opts & OPT_c) {
@@ -181,9 +176,9 @@ int touch_main(int argc UNUSED_PARAM, char **argv)
 				/* Try to create the file */
 				fd = open(*argv, O_RDWR | O_CREAT, 0666);
 				if (fd >= 0) {
+					if (opts & (OPT_r|OPT_d|OPT_t))
+						futimens(fd, timebuf);
 					xclose(fd);
-					if (reference_file || date_str)
-						utimes(*argv, timebuf);
 					continue;
 				}
 			}
