@@ -25,6 +25,12 @@
 	" (default "CONFIG_FEATURE_DEFAULT_PASSWD_ALGO")"
 #endif
 
+#if ENABLE_FEATURE_HWCLOCK_ADJTIME_FHS
+# define ADJTIME_PATH "/var/lib/hwclock/adjtime"
+#else
+# define ADJTIME_PATH "/etc/adjtime"
+#endif
+
 INSERT
 
 #define busybox_notes_usage \
