@@ -64,8 +64,8 @@ static void fprime_select(byte *dst, const byte *zero, const byte *one, byte con
 #endif
 
 static void fe_select(byte *dst,
-		   const byte *zero, const byte *one,
-		   byte condition)
+		const byte *zero, const byte *one,
+		byte condition)
 {
 	const byte mask = -condition;
 	int i;
@@ -108,26 +108,26 @@ static void raw_try_sub(byte *x, const byte *p)
 #if 0 //UNUSED
 static int prime_msb(const byte *p)
 {
-    int i;
-    byte x;
-    int shift = 1;
-    int z     = F25519_SIZE - 1;
+	int i;
+	byte x;
+	int shift = 1;
+	int z     = F25519_SIZE - 1;
 
-   /*
-       Test for any hot bits.
-       As soon as one instance is encountered set shift to 0.
-    */
+	/*
+	    Test for any hot bits.
+	    As soon as one instance is encountered set shift to 0.
+	 */
 	for (i = F25519_SIZE - 1; i >= 0; i--) {
-        shift &= ((shift ^ ((-p[i] | p[i]) >> 7)) & 1);
-        z -= shift;
-    }
+		shift &= ((shift ^ ((-p[i] | p[i]) >> 7)) & 1);
+		z -= shift;
+	}
 	x = p[z];
 	z <<= 3;
-    shift = 1;
-    for (i = 0; i < 8; i++) {
-        shift &= ((-(x >> i) | (x >> i)) >> (7 - i) & 1);
-        z += shift;
-    }
+	shift = 1;
+	for (i = 0; i < 8; i++) {
+		shift &= ((-(x >> i) | (x >> i)) >> (7 - i) & 1);
+		z += shift;
+	}
 
 	return z - 1;
 }
@@ -163,11 +163,11 @@ static void fprime_mul(byte *r, const byte *a, const byte *b,
 		const byte bit = (b[i >> 3] >> (i & 7)) & 1;
 		byte plusa[F25519_SIZE];
 
-	    for (j = 0; j < F25519_SIZE; j++) {
-		    c |= ((word16)r[j]) << 1;
-		    r[j] = (byte)c;
-		    c >>= 8;
-	    }
+		for (j = 0; j < F25519_SIZE; j++) {
+			c |= ((word16)r[j]) << 1;
+			r[j] = (byte)c;
+			c >>= 8;
+		}
 		raw_try_sub(r, modulus);
 
 		fprime_copy(plusa, r);
@@ -315,7 +315,7 @@ static void fe_mul__distinct(byte *r, const byte *a, const byte *b)
 
 		for (; j < F25519_SIZE; j++)
 			c += ((word32)a[j]) *
-			     ((word32)b[i + F25519_SIZE - j]) * 38;
+				((word32)b[i + F25519_SIZE - j]) * 38;
 
 		r[i] = c;
 	}
@@ -383,12 +383,10 @@ static void fe_inv__distinct(byte *r, const byte *x)
 	 * to avoid copying temporaries.
 	 */
 
-	/* 1 1 */
-	fe_mul__distinct(s, x, x);
-	fe_mul__distinct(r, s, x);
+	lm_copy(r, x);
 
-	/* 1 x 248 */
-	for (i = 0; i < 248; i++) {
+	/* 1, 1 x 249 */
+	for (i = 0; i < 249; i++) {
 		fe_mul__distinct(s, r, r);
 		fe_mul__distinct(r, s, x);
 	}
@@ -403,13 +401,11 @@ static void fe_inv__distinct(byte *r, const byte *x)
 	/* 0 */
 	fe_mul__distinct(r, s, s);
 
-	/* 1 */
-	fe_mul__distinct(s, r, r);
-	fe_mul__distinct(r, s, x);
-
-	/* 1 */
-	fe_mul__distinct(s, r, r);
-	fe_mul__distinct(r, s, x);
+	/* 1, 1 */
+	for (i = 0; i < 2; i++) {
+		fe_mul__distinct(s, r, r);
+		fe_mul__distinct(r, s, x);
+	}
 }
 
 #if 0 //UNUSED
@@ -435,12 +431,10 @@ static void exp2523(byte *r, const byte *x, byte *s)
 	 *     111111... 01
 	 */
 
-	/* 1 1 */
-	fe_mul__distinct(r, x, x);
-	fe_mul__distinct(s, r, x);
+	lm_copy(s, x);
 
-	/* 1 x 248 */
-	for (i = 0; i < 248; i++) {
+	/* 1, 1 x 249 */
+	for (i = 0; i < 249; i++) {
 		fe_mul__distinct(r, s, s);
 		fe_mul__distinct(s, r, x);
 	}
@@ -480,9 +474,9 @@ static void fe_sqrt(byte *r, const byte *a)
 
 /* Differential addition */
 static void xc_diffadd(byte *x5, byte *z5,
-		       const byte *x1, const byte *z1,
-		       const byte *x2, const byte *z2,
-		       const byte *x3, const byte *z3)
+		const byte *x1, const byte *z1,
+		const byte *x2, const byte *z2,
+		const byte *x3, const byte *z3)
 {
 	/* Explicit formulas database: dbl-1987-m3
 	 *
@@ -522,7 +516,7 @@ static void xc_diffadd(byte *x5, byte *z5,
 
 /* Double an X-coordinate */
 static void xc_double(byte *x3, byte *z3,
-		      const byte *x1, const byte *z1)
+		const byte *x1, const byte *z1)
 {
 	/* Explicit formulas database: dbl-1987-m
 	 *
@@ -550,11 +544,14 @@ static void xc_double(byte *x3, byte *z3,
 	fe_mul_c(z3, x1sq, 4);
 }
 
-void FAST_FUNC curve25519(byte *result, const byte *e, const byte *q)
+static void curve25519(byte *result, const byte *e, const byte *q)
 {
 	int i;
 
 	struct {
+		/* for bbox's special case of q == NULL meaning "use basepoint" */
+		/*static const*/ uint8_t basepoint9[CURVE25519_KEYSIZE]; // = {9};
+
 		/* from wolfssl-3.15.3/wolfssl/wolfcrypt/fe_operations.h */
 		/*static const*/ byte f25519_one[F25519_SIZE]; // = {1};
 
@@ -565,6 +562,7 @@ void FAST_FUNC curve25519(byte *result, const byte *e, const byte *q)
 		byte xm1[F25519_SIZE]; // = {1};
 		byte zm1[F25519_SIZE]; // = {0};
 	} z;
+#define basepoint9 z.basepoint9
 #define f25519_one z.f25519_one
 #define xm         z.xm
 #define zm         z.zm
@@ -575,6 +573,11 @@ void FAST_FUNC curve25519(byte *result, const byte *e, const byte *q)
 	zm[0] = 1;
 	xm1[0] = 1;
 
+	if (!q) {
+		basepoint9[0] = 9;
+		q = basepoint9;
+	}
+
 	/* Note: bit 254 is assumed to be 1 */
 	lm_copy(xm, q);
 
@@ -605,3 +608,23 @@ void FAST_FUNC curve25519(byte *result, const byte *e, const byte *q)
 	fe_mul__distinct(result, zm1, xm);
 	fe_normalize(result);
 }
+
+/* interface to bbox's TLS code: */
+
+void FAST_FUNC curve_x25519_compute_pubkey_and_premaster(
+		uint8_t *pubkey, uint8_t *premaster,
+		const uint8_t *peerkey32)
+{
+	uint8_t privkey[CURVE25519_KEYSIZE]; //[32]
+
+	/* Generate random private key, see RFC 7748 */
+	tls_get_random(privkey, sizeof(privkey));
+	privkey[0] &= 0xf8;
+	privkey[CURVE25519_KEYSIZE-1] = ((privkey[CURVE25519_KEYSIZE-1] & 0x7f) | 0x40);
+
+	/* Compute public key */
+	curve25519(pubkey, privkey, NULL /* "use base point of x25519" */);
+
+	/* Compute premaster using peer's public key */
+	curve25519(premaster, privkey, peerkey32);
+}
